from enum import Enum
from typing import List, Optional

from pydantic import BaseModel


class DayInterval(str, Enum):
    D1 = "D1"
    W1 = "W1"


class IntraDayInterval(str, Enum):
    M1 = "M1"
    M5 = "M5"
    M15 = "M15"
    H1 = "H1"


class BaseCandlesRequestModel(BaseModel):
    securityBoard: str
    securityCode: str
    intervalFrom: Optional[str]
    intervalTo: Optional[str]
    count: Optional[int]


class DayCandlesRequestModel(BaseCandlesRequestModel):
    timeFrame: DayInterval


class IntraDayCandlesRequestModel(BaseCandlesRequestModel):
    timeFrame: IntraDayInterval


class Decimal(BaseModel):
    num: int
    scale: int


class BaseCandleModel(BaseModel):
    open: Decimal
    close: Decimal
    high: Decimal
    low: Decimal
    volume: int


class DayCandle(BaseCandleModel):
    date: str


class CandlesResponse(BaseModel):
    candles: List[DayCandle]


class DayCandlesResponse(BaseModel):
    data: CandlesResponse


class IntraDayCandle(BaseCandleModel):
    timestamp: str


class InDayCandlesResponse(BaseModel):
    candles: List[IntraDayCandle]


class IntraDayCandlesResponse(BaseModel):
    data: InDayCandlesResponse



from enum import Enum
from typing import Optional

from pydantic import BaseModel


class BaseErrorModel(BaseModel):
    code: str
    message: str
    data: dict | None = None


class ErrorBodyModel(BaseModel):
    error: BaseErrorModel


class Market(str, Enum):
    Stock = "Stock"
    Forts = "Forts"
    Spbex = "Spbex"
    Mma = "Mma"
    Ets = "Ets"
    Bonds = "Bonds"
    Options = "Options"



import os

from finam_trade_api.client import Client


async def main():
    client = Client(os.getenv("TOKEN"))
    return await client.access_tokens.check_token()


if __name__ == "__main__":
    import asyncio

    print(asyncio.run(main()))



import pytest
from unittest.mock import patch, AsyncMock
from finam_trade_api.base_client.base import BaseClient


@pytest.fixture
def base_client():
    token = "test_token"
    return BaseClient(token)


@pytest.mark.asyncio
async def test_base_client_initialization(base_client):
    assert base_client._token == "test_token"
    assert base_client._base_url == "https://trade-api.finam.ru"


def test_auth_headers(base_client):
    expected_headers = {"X-Api-Key": "test_token"}
    assert base_client._auth_headers == expected_headers


@pytest.mark.asyncio
@patch('aiohttp.ClientSession')
async def test_exec_request_success(mock_session, base_client):
    mock_response = AsyncMock()
    mock_response.status = 200
    mock_response.json = AsyncMock(return_value={"key": "value"})

    mock_session_instance = mock_session.return_value
    mock_session_instance.__aenter__.return_value = mock_session_instance
    mock_session_instance.request.return_value.__aenter__.return_value = mock_response

    response, success = await base_client._exec_request("get", "/test-url")
    assert success is True
    assert response == {"key": "value"}


@pytest.mark.asyncio
@patch('aiohttp.ClientSession')
async def test_exec_request_failure(mock_session, base_client):
    mock_response = AsyncMock()
    mock_response.status = 404
    mock_response.json = AsyncMock(return_value={"error": "not found"})

    mock_session_instance = mock_session.return_value
    mock_session_instance.__aenter__.return_value = mock_session_instance
    mock_session_instance.request.return_value.__aenter__.return_value = mock_response

    response, success = await base_client._exec_request("get", "/test-url")
    assert success is False
    assert response == {"error": "not found"}



from typing import List

from finam_trade_api.base_client import BaseClient
from finam_trade_api.candles.model import (
    DayCandle,
    DayCandlesRequestModel,
    DayCandlesResponse,
    IntraDayCandle,
    IntraDayCandlesRequestModel,
    IntraDayCandlesResponse,
)
from finam_trade_api.exceptions import FinamTradeApiError
from finam_trade_api.models import ErrorBodyModel


class CandlesClient(BaseClient):
    def __init__(self, token: str):
        super().__init__(token)
        self._day_url = "/api/v1/day-candles"
        self._in_day_url = "/api/v1/intraday-candles"

    async def get_day_candles(self, params: DayCandlesRequestModel) -> List[DayCandle]:
        p = {
            "securityBoard": params.securityBoard,
            "securityCode": params.securityCode,
            "timeFrame": params.timeFrame.value,
            "interval.From": params.intervalFrom,
            "interval.To": params.intervalTo,
            "interval.Count": params.count,
        }

        response, ok = await self._exec_request(
            self.RequestMethod.GET,
            self._day_url,
            params={k: v for k, v in p.items() if v is not None},
        )
        if not ok:
            err = ErrorBodyModel(**response)
            raise FinamTradeApiError(f"{err.error.code} | {err.error.data} | {err.error.message}")
        return DayCandlesResponse(**response).data.candles

    async def get_in_day_candles(self, params: IntraDayCandlesRequestModel) -> List[IntraDayCandle]:
        p = {
            "securityBoard": params.securityBoard,
            "securityCode": params.securityCode,
            "timeFrame": params.timeFrame.value,
            "interval.From": params.intervalFrom,
            "interval.To": params.intervalTo,
            "interval.Count": params.count,
        }

        response, ok = await self._exec_request(
            self.RequestMethod.GET,
            self._in_day_url,
            params={k: v for k, v in p.items() if v is not None},
        )
        if not ok:
            err = ErrorBodyModel(**response)
            raise FinamTradeApiError(f"{err.error.code} | {err.error.data} | {err.error.message}")
        return IntraDayCandlesResponse(**response).data.candles



import os

from finam_trade_api.client import Client
from finam_trade_api.order.model import (
    BoardType,
    CreateOrderRequestModel,
    CreateStopOrderRequestModel,
    DelOrderModel,
    OrdersRequestModel,
    OrderType,
    PropertyType,
    StopLossModel,
    StopQuantity,
    StopQuantityUnits,
    TakeProfitModel
)

token = os.getenv("TOKEN", "")
client_id = os.getenv("CLIENT_ID", "")
client = Client(token)


async def create_order():
    payload = CreateOrderRequestModel(
        clientId=client_id,
        securityBoard=BoardType.TQBR,
        securityCode="ALRS",
        buySell=OrderType.Buy,
        quantity=1,
        price=72.23,
        property=PropertyType.PutInQueue,
        condition=None,
        validateBefore=None,
    )
    return await client.orders.create_order(payload)


async def get_orders():
    params = OrdersRequestModel(
        clientId=client_id,
        includeActive="true",
        includeMatched="true",
    )
    return await client.orders.get_orders(params)


async def del_order(transaction_id: int):
    params = DelOrderModel(
        clientId=client_id,
        transactionId=transaction_id
    )
    return await client.orders.del_order(params)


async def create_stop_order(transaction_id: int):
    payload = CreateStopOrderRequestModel(
        clientId=client_id,
        securityBoard=BoardType.TQBR,
        securityCode="ALRS",
        buySell=OrderType.Sell,
        linkOrder=transaction_id,
        stopLoss=StopLossModel(
            activationPrice=71.80,
            marketPrice=True,
            quantity=StopQuantity(
                value=1,
                units=StopQuantityUnits.Lots,
            )
        ),
        takeProfit=TakeProfitModel(
            activationPrice=72.30,
            marketPrice=True,
            quantity=StopQuantity(
                value=1,
                units=StopQuantityUnits.Lots,
            )
        ),
    )
    return await client.orders.create_stop_order(payload)



import pytest
from unittest.mock import AsyncMock, patch
from finam_trade_api.order.order import OrderClient
from finam_trade_api.order.model import (
    CreateOrderRequestModel,
    CreateOrderResponseModel,
    DelOrderModel,
    DelOrderResponseModel,
    OrdersRequestModel,
    OrdersResponseModel,
)


class TestOrderClient:

    @pytest.fixture
    def client(self):
        return OrderClient(token="mock_token")

    @pytest.fixture
    def mock_exec_request(self):
        with patch.object(OrderClient, '_exec_request', new_callable=AsyncMock) as mock:
            yield mock

    @pytest.mark.asyncio
    async def test_get_orders(self, client, mock_exec_request):
        mock_response = {"data": {"clientId": "clientId", "orders": []}}
        mock_exec_request.return_value = (mock_response, True)
        params = OrdersRequestModel(clientId="clientId")
        response = await client.get_orders(params)
        assert response == OrdersResponseModel(**mock_response)
        mock_exec_request.assert_called_once_with(
            client.RequestMethod.GET,
            client._order_url,
            params=params.model_dump(exclude_none=True)
        )
        assert response.data.clientId == "clientId"

    @pytest.mark.asyncio
    async def test_create_order(self, client, mock_exec_request):
        mock_response = {"data": {"clientId": "123", "transactionId": 1, "securityCode": "code"}}
        mock_exec_request.return_value = (mock_response, True)
        payload = CreateOrderRequestModel(
            **{
                "clientId": "clientId",
                "securityBoard": "TQBR",
                "securityCode": "code",
                "buySell": "Sell",
                "quantity": 1,
                "property": "PutInQueue"
            }
        )
        response = await client.create_order(payload)
        assert response == CreateOrderResponseModel(**mock_response)
        mock_exec_request.assert_called_once_with(
            client.RequestMethod.POST,
            client._order_url,
            payload.model_dump(exclude_none=True)
        )

    @pytest.mark.asyncio
    async def test_del_order(self, client, mock_exec_request):
        mock_response = {"data": {"clientId": "clientId", "transactionId": 2}}
        mock_exec_request.return_value = (mock_response, True)
        params = DelOrderModel(
            **{
                "clientId": "clientId", "transactionId": 1
            }
        )
        response = await client.del_order(params)
        assert response == DelOrderResponseModel(**mock_response)
        mock_exec_request.assert_called_once_with(
            client.RequestMethod.DELETE,
            client._order_url,
            params=params.model_dump()
        )



from finam_trade_api.access import TokenClient
from finam_trade_api.candles import CandlesClient
from finam_trade_api.order import OrderClient
from finam_trade_api.portfolio import PortfolioClient
from finam_trade_api.securities import SecurityClient


class Client:
    def __init__(self, token: str):
        self.portfolio = PortfolioClient(token)
        self.securities = SecurityClient(token)
        self.orders = OrderClient(token)
        self.access_tokens = TokenClient(token)
        self.candles = CandlesClient(token)



import pytest
from unittest.mock import AsyncMock, patch
from finam_trade_api.candles.candles import CandlesClient
from finam_trade_api.candles.model import (
    DayCandle,
    DayCandlesRequestModel,
    DayCandlesResponse,
    IntraDayCandle,
    IntraDayCandlesRequestModel,
    IntraDayCandlesResponse,
)
from finam_trade_api.exceptions import FinamTradeApiError
from finam_trade_api.models import ErrorBodyModel


@pytest.fixture
def candles_client():
    token = "sample_token"
    return CandlesClient(token)


@pytest.mark.asyncio
async def test_candles_client_initialization(candles_client):
    assert candles_client._day_url == "/api/v1/day-candles"
    assert candles_client._in_day_url == "/api/v1/intraday-candles"


@pytest.mark.asyncio
async def test_get_day_candles_success(candles_client):
    params = DayCandlesRequestModel(
        securityBoard="board",
        securityCode="code",
        timeFrame="D1",
        intervalFrom="2023-01-01",
        intervalTo="2023-01-02",
        count=10
    )
    response_data = {
        "data": {
            "candles": [
                {
                    "date": "2020-02-02",
                    "open": {"num": 100, "scale": 0},
                    "close": {"num": 110, "scale": 0},
                    "high": {"num": 115, "scale": 0},
                    "low": {"num": 95, "scale": 0},
                    "volume": 1000
                }
            ]
        }
    }
    day_candles_response = DayCandlesResponse(**response_data)

    with patch.object(candles_client, '_exec_request', new=AsyncMock(return_value=(response_data, True))):
        response = await candles_client.get_day_candles(params)
        assert response == day_candles_response.data.candles


@pytest.mark.asyncio
async def test_get_day_candles_error(candles_client):
    params = DayCandlesRequestModel(
        securityBoard="board",
        securityCode="code",
        timeFrame="D1",
        intervalFrom="2023-01-01",
        intervalTo="2023-01-02",
        count=10
    )
    error_data = {
        "error": {
            "code": "invalid_request",
            "message": "Invalid request parameters"
        }
    }
    error_response = ErrorBodyModel(**error_data)

    with patch.object(candles_client, '_exec_request', new=AsyncMock(return_value=(error_data, False))):
        with pytest.raises(FinamTradeApiError) as excinfo:
            await candles_client.get_day_candles(params)
        assert str(
            excinfo.value) == f"{error_response.error.code} | {error_response.error.data} | {error_response.error.message}"


@pytest.mark.asyncio
async def test_get_in_day_candles_success(candles_client):
    params = IntraDayCandlesRequestModel(
        securityBoard="board",
        securityCode="code",
        timeFrame="M5",
        intervalFrom="2023-01-01",
        intervalTo="2023-01-02",
        count=10
    )
    response_data = {
        "data": {
            "candles": [
                {
                    "timestamp": "2020-02-02 00:00:00",
                    "open": {"num": 100, "scale": 0},
                    "close": {"num": 110, "scale": 0},
                    "high": {"num": 115, "scale": 0},
                    "low": {"num": 95, "scale": 0},
                    "volume": 1000
                }
            ]
        }
    }
    in_day_candles_response = IntraDayCandlesResponse(**response_data)

    with patch.object(candles_client, '_exec_request', new=AsyncMock(return_value=(response_data, True))):
        response = await candles_client.get_in_day_candles(params)
        assert response == in_day_candles_response.data.candles


@pytest.mark.asyncio
async def test_get_in_day_candles_error(candles_client):
    params = IntraDayCandlesRequestModel(
        securityBoard="board",
        securityCode="code",
        timeFrame="M1",
        intervalFrom="2023-01-01",
        intervalTo="2023-01-02",
        count=10
    )
    error_data = {
        "error": {
            "code": "invalid_request",
            "message": "Invalid request parameters"
        }
    }
    error_response = ErrorBodyModel(**error_data)

    with patch.object(candles_client, '_exec_request', new=AsyncMock(return_value=(error_data, False))):
        with pytest.raises(FinamTradeApiError) as excinfo:
            await candles_client.get_in_day_candles(params)
        assert str(
            excinfo.value) == f"{error_response.error.code} | {error_response.error.data} | {error_response.error.message}"


from enum import Enum
from typing import List, Optional

from pydantic import BaseModel

from finam_trade_api.models import Market


class OrderType(str, Enum):
    Sell = "Sell"
    Buy = "Buy"


class ConditionType(str, Enum):
    Bid = "Bid"  # лучшая цена покупки
    BidOrLast = "BidOrLast"  # лучшая цена покупки или сделка по заданной цене и выше
    Ask = "Ask"  # лучшая цена продажи
    AskOrLast = "AskOrLast"  # лучшая цена продажи или сделка по заданной цене и ниже
    CovDown = "CovDown"  # обеспеченность ниже заданной
    CovUp = "CovUp"  # обеспеченность выше заданной
    LastUp = "LastUp"  # сделка на рынке по заданной цене или выше
    LastDown = "LastDown"  # сделка на рынке по заданной цене или ниже


class PropertyType(str, Enum):
    PutInQueue = "PutInQueue"  # неисполненная часть заявки помещается в очередь заявок биржи;
    CancelBalance = "CancelBalance"  # неисполненная часть заявки снимается с торгов;
    ImmOrCancel = "ImmOrCancel"  # сделки совершаются в том случае, если заявка может быть удовлетворена полностью


class ValidBeforeType(str, Enum):
    TillEndSession = "TillEndSession"  # до конца дня
    TillCancelled = "TillCancelled"  # до отмены
    ExactTime = "ExactTime"  # до указанной даты


class BoardType(str, Enum):
    Futures = "FUT"
    ZLG = "ZLG"
    TQBR = "TQBR"


class StopQuantityUnits(str, Enum):
    Percent = "Percent"
    Lots = "Lots"


class StopPriceUnits(str, Enum):
    Percent = "Percent"
    Pips = "Pips"


class OrdersRequestModel(BaseModel):
    clientId: str
    includeMatched: Optional[str] = None
    includeCanceled: Optional[str] = None
    includeActive: Optional[str] = None


class Condition(BaseModel):
    type: ConditionType
    price: float  # цена активации
    time: Optional[str] = None


class ValidBefore(BaseModel):
    type: ValidBeforeType
    time: Optional[str] = None


class OrderStatus(str, Enum):
    NONE = "None"
    Active = "Active"
    Cancelled = "Cancelled"
    Matched = "Matched"


class Order(BaseModel):
    orderNo: int
    transactionId: int
    securityCode: Optional[str] = None
    clientId: Optional[str] = None
    status: OrderStatus
    buySell: OrderType
    createdAt: Optional[str] = None
    acceptedAt: Optional[str] = None
    price: float = 0  # В последующих версиях API поле для рыночных заявок будет равно null, а не 0.
    quantity: int
    balance: int
    message: Optional[str] = None
    currency: Optional[str] = None
    condition: Optional[Condition] = None
    validBefore: Optional[ValidBefore] = None
    securityBoard: Optional[str] = None
    market: Market


class OrdersResponseData(BaseModel):
    clientId: str
    orders: List[Order]


class OrdersResponseModel(BaseModel):
    data: OrdersResponseData


class StopOrder(BaseModel):
    stopId: int
    securityCode: str
    securityBoard: BoardType
    market: Market
    clientId: str
    buySell: OrderType
    expirationDate: Optional[str] = None
    linkOrder: int
    validBefore: Optional[ValidBefore] = None
    status: OrderStatus
    message: Optional[str] = None
    orderNo: int
    tradeNo: int
    acceptedAt: Optional[str] = None
    canceledAt: Optional[str] = None
    currency: Optional[str] = None
    takeProfitExtremum: float
    takeProfitLevel: float
    stopLoss: Optional["StopLossModel"] = None
    takeProfit: Optional["TakeProfitModel"] = None


class StopOrdersResponseData(BaseModel):
    clientId: str
    stops: List[StopOrder]


class StopOrdersResponseModel(BaseModel):
    data: StopOrdersResponseData


class CreatedOrder(BaseModel):
    clientId: str
    transactionId: int
    securityCode: str


class CreateOrderResponseModel(BaseModel):
    data: CreatedOrder


class CreateOrderRequestModel(BaseModel):
    clientId: str
    securityBoard: BoardType
    securityCode: str
    buySell: OrderType
    quantity: int
    useCredit: bool = False
    price: float = 0  # В последующих версиях API поле для рыночных заявок будет равно null, а не 0.
    property: PropertyType
    condition: Optional[Condition] = None
    validBefore: Optional[ValidBefore] = None


class StopQuantity(BaseModel):
    value: float  # Значение объема.
    units: StopQuantityUnits  # Единицы объема стоп-заявки.


class StopLossModel(BaseModel):
    activationPrice: float
    price: float = 0
    marketPrice: bool
    quantity: StopQuantity
    time: int = 0  # Защитное время, сек.
    useCredit: bool = False


class StopPrice(BaseModel):
    value: float  # Значение объема.
    units: StopPriceUnits  # Единицы объема стоп-заявки.


class TakeProfitModel(BaseModel):
    activationPrice: float
    correctionPrice: Optional[StopPrice] = None
    spreadPrice: Optional[StopPrice] = None
    marketPrice: bool
    quantity: StopQuantity
    time: int = 0  # Защитное время, сек.
    useCredit: bool = False


class CreateStopOrderRequestModel(BaseModel):
    clientId: str
    securityBoard: BoardType
    securityCode: str
    buySell: OrderType
    stopLoss: StopLossModel
    takeProfit: TakeProfitModel
    expirationDate: Optional[str] = None
    linkOrder: Optional[int] = None
    validBefore: Optional[ValidBefore] = None


class CreateStopOrderData(BaseModel):
    clientId: str
    stopId: int
    securityCode: Optional[str] = None
    securityBoard: Optional[str] = None


class CreateStopOrderResponseModel(BaseModel):
    data: CreateStopOrderData


class DelOrderModel(BaseModel):
    clientId: str
    transactionId: int


class DelOrderResponseModel(BaseModel):
    data: DelOrderModel


class DelStopOrderRequestModel(BaseModel):
    clientId: str
    stopOrderId: int


class DelStopOrderData(BaseModel):
    clientId: str
    stopOrderId: int


class DelStopOrderResponseModel(BaseModel):
    data: DelStopOrderData



from finam_trade_api.base_client import BaseClient
from finam_trade_api.exceptions import FinamTradeApiError
from finam_trade_api.models import ErrorBodyModel
from finam_trade_api.order.model import (
    CreateOrderRequestModel,
    CreateOrderResponseModel,
    CreateStopOrderRequestModel,
    CreateStopOrderResponseModel,
    DelOrderModel,
    DelOrderResponseModel,
    DelStopOrderRequestModel,
    OrdersRequestModel,
    OrdersResponseModel,
    StopOrdersResponseModel,
)
from finam_trade_api.order.model import DelStopOrderResponseModel as DelStopOrderResponse


class OrderClient(BaseClient):
    def __init__(self, token: str):
        super().__init__(token)
        self._order_url = "/api/v1/orders"
        self._stop_order_url = "/api/v1/stops"

    async def get_orders(self, params: OrdersRequestModel) -> OrdersResponseModel:
        response, ok = await self._exec_request(
            self.RequestMethod.GET,
            self._order_url,
            params=params.model_dump(exclude_none=True)
        )
        if not ok:
            err = ErrorBodyModel(**response)
            raise FinamTradeApiError(f"{err.error.code} | {err.error.data} | {err.error.message}")
        return OrdersResponseModel(**response)

    async def create_order(self, payload: CreateOrderRequestModel) -> CreateOrderResponseModel:
        response, ok = await self._exec_request(
            self.RequestMethod.POST,
            self._order_url,
            payload.model_dump(exclude_none=True)
        )
        if not ok:
            err = ErrorBodyModel(**response)
            raise FinamTradeApiError(f"{err.error.code} | {err.error.data} | {err.error.message}")
        return CreateOrderResponseModel(**response)

    async def del_order(self, params: DelOrderModel) -> DelOrderResponseModel:
        response, ok = await self._exec_request(
            self.RequestMethod.DELETE,
            self._order_url,
            params=params.model_dump()
        )
        if not ok:
            err = ErrorBodyModel(**response)
            raise FinamTradeApiError(f"{err.error.code} | {err.error.data} | {err.error.message}")
        return DelOrderResponseModel(**response)

    async def get_stop_orders(self, params: OrdersRequestModel) -> StopOrdersResponseModel:
        response, ok = await self._exec_request(
            self.RequestMethod.GET,
            self._stop_order_url,
            params.model_dump(exclude_none=True)
        )
        if not ok:
            err = ErrorBodyModel(**response)
            raise FinamTradeApiError(f"{err.error.code} | {err.error.data} | {err.error.message}")
        return StopOrdersResponseModel(**response)

    async def create_stop_order(self, payload: CreateStopOrderRequestModel) -> CreateStopOrderResponseModel:
        response, ok = await self._exec_request(
            self.RequestMethod.POST,
            self._stop_order_url,
            payload.model_dump(exclude_none=True)
        )
        if not ok:
            err = ErrorBodyModel(**response)
            raise FinamTradeApiError(f"{err.error.code} | {err.error.data} | {err.error.message}")
        return CreateStopOrderResponseModel(**response)

    async def del_stop_order(self, params: DelStopOrderRequestModel) -> DelStopOrderResponse:
        response, ok = await self._exec_request(
            self.RequestMethod.DELETE,
            self._stop_order_url,
            params.model_dump()
        )
        if not ok:
            err = ErrorBodyModel(**response)
            raise FinamTradeApiError(f"{err.error.code} | {err.error.data} | {err.error.message}")
        return DelStopOrderResponse(**response)



from finam_trade_api.base_client import BaseClient
from finam_trade_api.exceptions import FinamTradeApiError
from finam_trade_api.models import ErrorBodyModel
from finam_trade_api.portfolio.model import PortfolioRequestModel, PortfolioResponseModel


class PortfolioClient(BaseClient):
    def __init__(self, token: str):
        super().__init__(token)
        self._url = "/api/v1/portfolio"

    async def get_portfolio(self, params: PortfolioRequestModel) -> PortfolioResponseModel:
        response, ok = await self._exec_request(
            self.RequestMethod.GET,
            self._url,
            params={
                "clientId": params.clientId,
                "content.IncludeCurrencies": params.includeCurrencies,
                "content.IncludeMoney": params.includeMoney,
                "content.IncludePositions": params.includePositions,
                "content.IncludeMaxBuySell": params.includeMaxBuySell,
            }
        )
        if not ok:
            err = ErrorBodyModel(**response)
            raise FinamTradeApiError(f"{err.error.code} | {err.error.data} | {err.error.message}")
        return PortfolioResponseModel(**response)



import pytest
from unittest.mock import AsyncMock, patch
from finam_trade_api.access.access_token import TokenClient
from finam_trade_api.access.model import TokenResponseModel
from finam_trade_api.exceptions import FinamTradeApiError
from finam_trade_api.models import ErrorBodyModel


@pytest.fixture
def token_client():
    token = "sample_token"
    return TokenClient(token)


@pytest.mark.asyncio
async def test_token_client_initialization(token_client):
    assert token_client._url == "/api/v1/access-tokens"


@pytest.mark.asyncio
async def test_check_token_success(token_client):
    response_data = {
        "data": {
            "id": 12345,
        }
    }
    token_response = TokenResponseModel(**response_data)

    with patch.object(token_client, '_exec_request', new=AsyncMock(return_value=(response_data, True))):
        response = await token_client.check_token()
        assert response == token_response


@pytest.mark.asyncio
async def test_check_token_error(token_client):
    error_data = {
        "error": {
            "code": "invalid_token",
            "message": "The token is invalid"
        }
    }
    error_response = ErrorBodyModel(**error_data)

    with patch.object(token_client, '_exec_request', new=AsyncMock(return_value=(error_data, False))):
        with pytest.raises(FinamTradeApiError) as excinfo:
            await token_client.check_token()
        assert str(
            excinfo.value) == f"{error_response.error.code} | {error_response.error.data} | {error_response.error.message}"



from abc import ABC
from enum import Enum
from typing import Any, Tuple

import aiohttp


class BaseClient(ABC):
    class RequestMethod(str, Enum):
        POST = "post"
        PUT = "put"
        GET = "get"
        DELETE = "delete"

    def __init__(self, token: str, url: str = "https://trade-api.finam.ru"):
        self._token = token
        self._base_url = url

    @property
    def _auth_headers(self):
        return {"X-Api-Key": self._token}

    async def _exec_request(self, method: str, url: str, payload=None, **kwargs) -> Tuple[Any, bool]:
        uri = f"{self._base_url}{url}"

        async with aiohttp.ClientSession(headers=self._auth_headers) as session:
            async with session.request(method, uri, json=payload, **kwargs) as response:
                if response.status != 200:
                    if response.content_type != "application/json":
                        response.raise_for_status()
                    return await response.json(), False
                return await response.json(), True



from finam_trade_api.portfolio.model import (
    Currency,
    Money,
    PortfolioContent,
    PortfolioRequestModel,
    PortfolioRequestOptionalModel,
    PortfolioResponseData,
    PortfolioResponseModel,
    Position,
)
from finam_trade_api.portfolio.portfolio import PortfolioClient



from finam_trade_api.candles.candles import CandlesClient
from finam_trade_api.candles.model import (
    DayCandle,
    DayCandlesRequestModel,
    DayInterval,
    Decimal,
    IntraDayCandle,
    IntraDayCandlesRequestModel,
    IntraDayInterval,
)



import pytest
from finam_trade_api.access.access_token import TokenClient
from finam_trade_api.candles.candles import CandlesClient
from finam_trade_api.client import Client
from finam_trade_api.order.order import OrderClient
from finam_trade_api.portfolio.portfolio import PortfolioClient
from finam_trade_api.securities.securities import SecurityClient


@pytest.fixture
def client():
    token = "your_token_here"
    return Client(token)


def test_client_initialization(client):
    assert isinstance(client.portfolio, PortfolioClient)
    assert isinstance(client.securities, SecurityClient)
    assert isinstance(client.orders, OrderClient)
    assert isinstance(client.access_tokens, TokenClient)
    assert isinstance(client.candles, CandlesClient)



from typing import Optional

from finam_trade_api.base_client import BaseClient
from finam_trade_api.exceptions import FinamTradeApiError
from finam_trade_api.models import ErrorBodyModel
from finam_trade_api.securities.model import SecurityResponseModel


class SecurityClient(BaseClient):
    def __init__(self, token: str):
        super().__init__(token)
        self._url = "/api/v1/securities"
        self._params: dict = {}

    async def get_data(
            self,
            code: Optional[str] = None,
            board: Optional[str] = None,
    ) -> SecurityResponseModel:
        if board:
            self._params["board"] = board
        if code:
            self._params["seccode"] = code

        response, ok = await self._exec_request(self.RequestMethod.GET, self._url, params=self._params)
        if not ok:
            err = ErrorBodyModel(**response)
            raise FinamTradeApiError(f"{err.error.code} | {err.error.data} | {err.error.message}")
        return SecurityResponseModel(**response)



from typing import List, Optional

from pydantic import BaseModel

from finam_trade_api.models import Market


class Security(BaseModel):
    code: str
    board: str
    market: Market
    decimals: int
    lotSize: int
    minStep: int
    currency: Optional[str] = None
    shortName: Optional[str] = None
    properties: int
    timeZoneName: Optional[str] = None
    bpCost: float
    accruedInterest: float
    priceSign: str
    ticker: Optional[str] = None
    lotDivider: int


class SecurityData(BaseModel):
    securities: List[Security]


class SecurityResponseModel(BaseModel):
    data: SecurityData



from typing import List

from pydantic import BaseModel

from finam_trade_api.models import Market


class PortfolioRequestOptionalModel(BaseModel):
    includeCurrencies: str = "true"
    includeMoney: str = "false"
    includeMaxBuySell: str = "true"
    includePositions: str = "true"


class PortfolioRequestModel(PortfolioRequestOptionalModel):
    clientId: str


class PortfolioContent(BaseModel):
    includeCurrencies: bool
    includeMoney: bool
    includePositions: bool
    includeMaxBuySell: bool


class Position(BaseModel):
    securityCode: str
    market: Market
    balance: int
    currentPrice: float
    equity: float
    averagePrice: float
    currency: str
    accumulatedProfit: float
    todayProfit: float
    unrealizedProfit: float
    profit: float
    maxBuy: int
    maxSell: int
    priceCurrency: str
    averagePriceCurrency: str
    averageRate: float


class Currency(BaseModel):
    name: str
    balance: float
    crossRate: float
    equity: float
    unrealizedProfit: float


class Money(BaseModel):
    market: str
    currency: str
    balance: float


class PortfolioResponseData(BaseModel):
    clientId: str
    content: PortfolioContent
    equity: float
    balance: float
    positions: List[Position]
    currencies: List[Currency]
    money: List[Money]


class PortfolioResponseModel(BaseModel):
    data: PortfolioResponseData



import pytest
from unittest.mock import AsyncMock, patch
from finam_trade_api.securities.securities import SecurityClient
from finam_trade_api.exceptions import FinamTradeApiError
from finam_trade_api.securities.model import SecurityResponseModel


@pytest.fixture
def security_client():
    return SecurityClient(token="test_token")


@pytest.fixture
def mock_response():
    return {
        "data": {
            "securities": [
                {
                    "code": "TEST_CODE",
                    "board": "TQBR",
                    "market": "Stock",
                    "decimals": 2,
                    "lotSize": 10,
                    "minStep": 1,
                    "properties": 1,
                    "bpCost": 10,
                    "accruedInterest": 10,
                    "priceSign": "priceSign",
                    "ticker": "SBER",
                    "lotDivider": 1
                }
            ],
        }
    }


@pytest.mark.asyncio
async def test_security_client_initialization(security_client):
    assert security_client._url == "/api/v1/securities"
    assert security_client._params == {}


@pytest.mark.asyncio
async def test_get_data_with_code(security_client, mock_response):
    with patch.object(security_client, '_exec_request', new=AsyncMock(return_value=(mock_response, True))):
        response = await security_client.get_data(code="TEST_CODE")
        assert response == SecurityResponseModel(**mock_response)
        assert security_client._params["seccode"] == "TEST_CODE"


@pytest.mark.asyncio
async def test_get_data_with_board(security_client, mock_response):
    with patch.object(security_client, '_exec_request', new=AsyncMock(return_value=(mock_response, True))):
        response = await security_client.get_data(board="TQBR")
        assert response == SecurityResponseModel(**mock_response)
        assert security_client._params["board"] == "TQBR"


@pytest.mark.asyncio
async def test_get_data_with_code_and_board(security_client, mock_response):
    with patch.object(security_client, '_exec_request', new=AsyncMock(return_value=(mock_response, True))):
        response = await security_client.get_data(code="TEST_CODE", board="TQBR")
        assert response == SecurityResponseModel(**mock_response)
        assert security_client._params["seccode"] == "TEST_CODE"
        assert security_client._params["board"] == "TQBR"


@pytest.mark.asyncio
async def test_get_data_error_handling(security_client):
    mock_error_response = {
        "error": {
            "code": "ERROR_CODE",
            "data": None,
            "message": "ERROR_MESSAGE"
        }
    }
    with patch.object(security_client, '_exec_request', new=AsyncMock(return_value=(mock_error_response, False))):
        with pytest.raises(FinamTradeApiError) as exc_info:
            await security_client.get_data(code="TEST_CODE")
        assert str(exc_info.value) == "ERROR_CODE | None | ERROR_MESSAGE"



from finam_trade_api.access.model import TokenResponseModel
from finam_trade_api.base_client.base import BaseClient
from finam_trade_api.exceptions import FinamTradeApiError
from finam_trade_api.models import ErrorBodyModel


class TokenClient(BaseClient):
    def __init__(self, token: str):
        super().__init__(token)
        self._url = "/api/v1/access-tokens"

    async def check_token(self) -> TokenResponseModel:
        response, ok = await self._exec_request(
            self.RequestMethod.GET,
            f"{self._url}/check",
        )
        if not ok:
            err = ErrorBodyModel(**response)
            raise FinamTradeApiError(f"{err.error.code} | {err.error.data} | {err.error.message}")
        return TokenResponseModel(**response)



import pytest
from unittest.mock import AsyncMock, patch
from finam_trade_api.portfolio.portfolio import PortfolioClient
from finam_trade_api.portfolio.model import PortfolioRequestModel, PortfolioResponseModel
from finam_trade_api.exceptions import FinamTradeApiError
from finam_trade_api.models import ErrorBodyModel


class TestPortfolioClient:

    @pytest.fixture
    def client(self):
        return PortfolioClient(token="dummy_token")

    @pytest.fixture
    def request_params(self):
        return PortfolioRequestModel(
            clientId="123",
            includeCurrencies="true",
            includeMoney="true",
            includePositions="true",
            includeMaxBuySell="true"
        )

    @pytest.fixture
    def successful_response(self):
        return {
            "data": {
                "clientId": "123",
                "content": {
                    "includeCurrencies": True,
                    "includeMoney": True,
                    "includePositions": True,
                    "includeMaxBuySell": True,
                },
                "equity": 10.10,
                "balance": 1000,
                "currencies": [],
                "money": [],
                "positions": [],
            }
        }

    @pytest.fixture
    def error_response(self):
        return {
            "error": {
                "code": "400",
                "data": None,
                "message": "Bad Request"
            }
        }

    @patch('finam_trade_api.portfolio.portfolio.PortfolioClient._exec_request', new_callable=AsyncMock)
    @pytest.mark.asyncio
    async def test_get_portfolio_success(self, mock_exec_request, client, request_params, successful_response):
        mock_exec_request.return_value = (successful_response, True)
        response = await client.get_portfolio(request_params)
        assert isinstance(response, PortfolioResponseModel)
        assert response.data.clientId == "123"

    @patch('finam_trade_api.portfolio.portfolio.PortfolioClient._exec_request', new_callable=AsyncMock)
    @pytest.mark.asyncio
    async def test_get_portfolio_error(self, mock_exec_request, client, request_params, error_response):
        mock_exec_request.return_value = (error_response, False)
        with pytest.raises(FinamTradeApiError) as exc_info:
            await client.get_portfolio(request_params)
        assert "400 | None | Bad Request" in str(exc_info.value)



import os

from finam_trade_api.client import Client
from finam_trade_api.candles.model import (
    DayCandlesRequestModel,
    DayInterval,
    IntraDayCandlesRequestModel,
    IntraDayInterval
)

token = os.getenv("TOKEN", "")


async def get_day_candles():
    client = Client(token)
    params = DayCandlesRequestModel(
        securityBoard="TQBR",
        securityCode="SBER",
        timeFrame=DayInterval.D1,
        intervalFrom="2023-06-05",
        intervalTo="2023-06-07",
    )
    return await client.candles.get_day_candles(params)


async def get_in_day_candles():
    client = Client(token)
    params = IntraDayCandlesRequestModel(
        securityBoard="TQBR",
        securityCode="SBER",
        timeFrame=IntraDayInterval.M1,
        intervalFrom="2023-06-07 08:33:52",
        count=10
    )
    return await client.candles.get_in_day_candles(params)


if __name__ == "__main__":
    import asyncio

    print(asyncio.run(get_day_candles()))

    print(asyncio.run(get_in_day_candles()))



import os

from finam_trade_api.client import Client

token = os.getenv("TOKEN", "")
client = Client(token)


async def get_data_by_code(code: str):
    return await client.securities.get_data(code)


if __name__ == "__main__":
    import asyncio

    print(asyncio.run(get_data_by_code("SBER")))



from finam_trade_api.order.model import (
    BoardType,
    Condition,
    ConditionType,
    CreatedOrder,
    CreateOrderRequestModel,
    CreateOrderResponseModel,
    CreateStopOrderData,
    CreateStopOrderRequestModel,
    CreateStopOrderResponseModel,
    DelOrderModel,
    DelOrderResponseModel,
    DelStopOrderData,
    DelStopOrderRequestModel,
    DelStopOrderResponseModel,
    Order,
    OrdersRequestModel,
    OrdersResponseData,
    OrdersResponseModel,
    OrderStatus,
    OrderType,
    PropertyType,
    StopLossModel,
    StopOrder,
    StopOrdersResponseData,
    StopOrdersResponseModel,
    StopPrice,
    StopPriceUnits,
    StopQuantity,
    StopQuantityUnits,
    TakeProfitModel,
    ValidBefore,
    ValidBeforeType,
)
from finam_trade_api.order.order import OrderClient
